'use strict';

const monitor = require('cf-monitor');
const Promise = require('bluebird');
const express = require('express');
const compression = require('compression');
const bodyParser = require('body-parser');
const methodOverride = require('method-override');
const cookieParser = require('cookie-parser');
const morgan = require('morgan');
const logger = require('cf-logs').Logger("codefresh:infra:express");
const api = require('../server/api');
const _ = require('lodash');
const expressValidator = require('express-validator');
const middlewares = require('./middleware');
const httpInfra = require('@codefresh-io/http-infra');
const CFError = require('cf-errors');


class Express {

    constructor() {
        this.expressApp; // the express app definition
        this.expressServer; // the express server instance
        this.healthy = true;
    }

    /**
     * starts the connection to mongo
     * @returns {*}
     */
    init(config) {
        return Promise.resolve()
            .then(() => {
                this.config = config;
                return this._create()
                    .then((expressApp) => {
                        this.expressApp = expressApp;
                        return this._start(this.expressApp);
                    })
                    .then((expressServer) => {
                        this.expressServer = expressServer;
                    });
            });
    }


    /**
     * stops the express app server
     * @returns {*}
     */
    stop() {
        return Promise.resolve()
            .then(() => {
                this.healthy = false;
                logger.info('Express health route marked as unhealthy');
            });
    }

    _create() {
        return Promise.resolve()
            .then(() => {
                const app = express();
                app.use(httpInfra.newDomainMiddleware());
                app.use(cookieParser());
                app.use(compression());
                app.use(bodyParser.urlencoded({ extended: true }));
                app.use(bodyParser.json());
                app.use(expressValidator([]));
                app.use(methodOverride());

                if (this.config.httpLogger) {
                    app.use(morgan(this.config.httpLogger.format, {
                        skip: (req, res) => {
                            if (this.config.httpLogger.level === 'debug') {
                                return false;
                            }
                            var code = res.statusCode;
                            return code < 400;
                        },
                        stream: {
                            write: (str) => {
                                logger.info(str.substring(0, str.length - 1));
                            }
                        }
                    }));
                }

                app.get('/api/ping', (req, res) => {
                    res.status(200).send();
                });

                app.get('/api/health', (req, res) => {
                    if (this.healthy) {
                        res.status(200).send();
                    } else {
                        res.status(400).send();
                    }
                });

                app.use((request, response, next) => {
                    const _decodeBase64 = (str) => Buffer.from(str, 'base64').toString();

                    const userHeader = request.headers['x-user-json-encoded'];
                    const userHeaderOld = request.headers['x-user-json'];

                    if (userHeader) {
                        request.user = JSON.parse(_decodeBase64(userHeader));
                    } else if (userHeaderOld) {
                        request.user = JSON.parse(userHeaderOld);
                    }

                    const accountHeader = request.headers['x-account-json-encoded'];
                    const accountHeaderOld = request.headers['x-account-json'];

                    if (accountHeader) {
                        request.account = JSON.parse(_decodeBase64(accountHeader));
                    } else if (accountHeaderOld) {
                        request.account = JSON.parse(accountHeaderOld);
                    }

                    next();
                });

                app.use((request, response, next) => {
                    const headers     = request.headers;
                    request.accountId = headers['x-account-id'];
                    request.userId    = headers['x-user-id'];
                    next();
                });

                app.use(middlewares.prepareKubeConfig);

                app.use(middlewares.resolveQueryString);

                api.createRoutes(app);

                // the last error handler
                app.use((err, req, res, next) => {
                    if (err instanceof CFError) {
                        if (!err.getFirstValue("recognized")) {
                            monitor.noticeError(err);
                        }
                    }
                    logger.error(err.stack);
                    if (res.headersSent) {
                        return next(err);
                    }

                    const statusCode = err.statusCode || 500;

                    res.status(statusCode).send({
                        message: err.message,
                        selector: _.get(req, 'query.selector')
                    });
                });

                return app;
            });

    }

    _start(app) {
        return new Promise((resolve, reject) => {

            const server = app.listen(this.config.port, (err) => {
                if (err) {
                    console.log(`Failed to load service with message ${err.message}`);
                    reject(err);
                } else {
                    console.log(`Express server listening on port ${this.config.port}, in mode ${this.config.env}`);
                    resolve(server);
                }
            });

        });
    }


}


module.exports = new Express();
