const Promise = require('bluebird');
const _ = require('lodash');

class BasePool {
    constructor({ Revision, Release }, { sources }) {
        this._Revision = Revision;
        this._Release = Release;

        this._sources = sources;
        this._releases = [];
        this._versions = {};
    }

    isReleaseExistInPool(name){
        return !!this.getReleaseFromPool(name);
    }

    needProcessRevision(actualVersion, revision) {
        return actualVersion < revision._version;
    }

    getReleaseFromPool(name){
        return this._releases.find(re => re.name === name);
    }

    addReleaseToPool(release){
        this._releases.push(release);
    }

    getAllReleases(){
        return this._releases;
    }

    createReleaseFromRevision(revision, tillerNamespace){
        return new this._Release(revision, tillerNamespace);
    }

    getIgnoredRevisions() {
        return this._versions;
    }

    /**
     *
     * @param exists - releases that we already have
     * @param actual - releases that was retrieved from cluster
     */
    getDeletedReleases(exists) {
        return _.difference(exists, _.uniq(this._getReleases(this._sources)));
    }

    prepareReleasesWithOptimization(actualVersions = {}) {
        return Promise.map(this._sources, async (source) => {
            const revision = await this._readRevisionDataFromSource(source, false);
            const { name } = revision;
            const namespace = source.getNamespace();
            const key = [name, namespace].join(':');
            const actualVersion = actualVersions[key] || -1;
            if (this.needProcessRevision(actualVersion, revision)) {
                await revision.init();
                if (!this.isReleaseExistInPool(name)) {
                    const release = this.createReleaseFromRevision(revision, source.getNamespace());
                    this.addReleaseToPool(release);
                } else {
                    const release = this.getReleaseFromPool(name);
                    release.addRevision(revision);
                }
            } else {
                if (!this._versions[key]) {
                    this._versions[key] = [];
                }
                this._versions[key].push(revision._version);
            }
        });
    }

    prepareReleases() {
        return Promise.map(this._sources, (source) => {
            return this._readRevisionDataFromSource(source)
                .then((revision) => {
                    const name = revision.name;
                    if (!this.isReleaseExistInPool(name)) {
                        const release = this.createReleaseFromRevision(revision, source.getNamespace());
                        this.addReleaseToPool(release);
                    } else {
                        const release = this.getReleaseFromPool(name);
                        release.addRevision(revision);
                    }
                })
                .catch(err => {
                    console.log(err);
                });
        });
    }


    _getReleases(sources) {
        // abstract
    }


    async _readRevisionDataFromSource(source, withInit = true) {
        const rawName = source.getMetadata().name;
        const labels = source.getLabels();
        const dataAsBase64 = source.getProperty('data.release');
        const revision = new this._Revision(Object.assign({
            rawName,
            dataAsBase64
        }, labels));

        if (withInit) {
            await revision.init();

        }
        return revision;
    }
}


module.exports = BasePool;
