'use strict';

const EventEmitter = require('events');
const _            = require('lodash');
const BaseSA       = require('./../../status.analyzer.base');
const eventes      = require('./project.internal.evenets');

/**
 * Mapping between events on the project and the flow that should be executed
 * Could be String or String[]
 * For now when adding new resource need to evaluate the project status again
 */
const eventsToExecutoresMap = {
    [eventes.pod.addedSet]: ['analyzeStatus', '_onPodSetAdded'],
    [eventes.service.added]: ['analyzeStatus', '_onServiceAdded'],
    [eventes.deployment.added]: 'analyzeStatus',
    [eventes.rs.addedSet]: 'analyzeStatus'
};

/**
 * Project is an abstraction of set of resources that together have applicative purpose
 * Project atm have to include Kubernetes Service
 */
class Project extends EventEmitter {
    constructor({ cluster, ns }) {
        super();
        this._selector  = cluster;
        this._ns        = ns;
        this._resources = {};
        this._status    = {
            type: BaseSA.types.unknown.status
        };
        /**
         * Metadata is an object that hold all the extra data about the project
         * selectors
         * labels
         * containers and images with statuses
         * @private
         */
        this._metadata = {};
        this._name;

        this._subscribeToEvents();
    }

    setName(name) {
        this._name = name;
    }

    /**
     * Get the project name
     * @return {*}
     */
    getName() {
        return this._name;
    }

    getNamespace() {
        return this._ns;
    }

    /**
     * Traverse all the resources to determine the status of the porject
     */
    analyzeStatus() {
        const { service, deployment, replicaSets, pods } = this._resources;

        const status = {
            type: BaseSA.types.unknown.status,
            resources: {
                svc: service && service.getEntityStatus(),
                deployment: deployment && deployment.getEntityStatus(),
                replicaSets: replicaSets && replicaSets.map(rs => rs.getEntityStatus()),
                pods: pods && pods.map(pod => pod.getEntityStatus())
            }
        };

        if (service && service.getEntityStatus().status !== BaseSA.types.suceess.status) {
            status.type    = BaseSA.types.failed;
            status.message = 'Kubernetes service not available';
        }

        if (_.size(replicaSets) > 1) {
            let rs = _.find(replicaSets, (obj) => {
                return obj.getEntityStatus().status !== BaseSA.types.suceess.status;
            });
            if (!rs) {
                rs = _.head(replicaSets);
            }
            const key         = rs.getLabelByKey('pod-template-hash');
            const matchedPods = _.filter(pods, (obj) => {
                return obj.getLabelByKey('pod-template-hash') === key;
            });
            const pod         = _.head(matchedPods);
            if (pod) {
                const podStatusObj = pod.getEntityStatus();
                status.type        = podStatusObj.status;
                status.message     = podStatusObj.message;
            } else if (deployment) {
                status.type = deployment.getEntityStatus().status;
            } else {
                status.type = rs.getEntityStatus().status;
            }

        } else {
            const pod = _.find(pods,
                (po) => po.getEntityStatus().status !== BaseSA.types.suceess.status);
            if (pod) {
                const podStatusObj = pod.getEntityStatus();
                status.type        = podStatusObj.status;
                status.message     = podStatusObj.message;
            } else {
                status.type = BaseSA.types.suceess.status;
            }
        }
        _.merge(this._status, status);
    }

    /**
     * Functions that related to kubernetes resources
     */

    setService(svc) {
        this._resources.service = svc;
        this.setName(svc.getName());
        this._updateMetadata({ projectSelectors: svc.getServiceSelector() });
        this.emit(eventes.service.added, svc);
    }

    setDeployment(deploy) {
        this._resources.deployment = deploy;
        this._updateProjectLabels(deploy.getLabels());
        this.emit(eventes.deployment.added, deploy);
    }

    setPods(pods) {
        if (!_.isArray(pods)) {
            pods = [pods];
        }
        this._resources.pods = pods;
        this.emit(eventes.pod.addedSet, pods);

    }

    setReplicaSets(rss) {
        if (!_.isArray(rss)) {
            rss = [rss];
        }
        this._resources.replicaSets = rss;
        this.emit(eventes.rs.addedSet, rss);
    }

    /**
     * Print functions
     */

    /**
     * Return minimal information about the project
     * @return {{name: *, selector: *, metadata: ({}|*), namespace: *, status: ({type: string, resources: {svc: *, deployment: *, replicaSets: *, pods: *}}|*|{type: string}), replicas: {running: number, total: number}}}
     */
    toMinifiedJson() {
        const res = {
            name: this._name,
            selector: this._selector,
            metadata: this._metadata,
            namespace: this._ns,
            status: this._status,

        };
        return res;
    }

    /**
     * Return the full information about the project, including the resources with full description
     * @return {{name: *, selector: *, metadata: ({}|*), namespace: *, resources: {}, status: ({type: string, resources: {svc: *, deployment: *, replicaSets: *, pods: *}}|*|{type: string})}}
     */
    toJson() {
        const res = {
            name: this._name,
            selector: this._selector,
            metadata: this._metadata,
            namespace: this._ns,
            resources: {},
            status: this._status
        };

        _.forOwn(this._resources, (value, key) => {
            if (_.isArray(value)) {
                res.resources[key] = [];
                _.forEach(value, (v) => res.resources[key].push(v.getFullData()));
            } else {
                res.resources[key] = value.getFullData();
            }
        });

        return res;
    }

    /**
     * Event handlers
     */

    _onPodSetAdded(pods) {
        const rss = _.get(this, '_resources.replicaSets', []);

        let containers = [];

        // pod can be created without rs or other asset
        if (!_.size(rss)) {
            const selectedPod = _.head(pods);
            containers        = selectedPod.getContainers();
        } else {
            rss.map((rs) => {
                const hash      = rs.getLabelByKey('pod-template-hash');
                const name      = rs.getName();
                const matchPods = _.filter(pods,
                    (pod) => pod.getLabelByKey('pod-template-hash') === hash);

                const selectedPod = _.head(matchPods);

                const podContainers = _.forEach(selectedPod.getContainers(),
                    (container) => _.merge(container, { rs: name }));
                containers          = _.concat(containers, podContainers);

            });
        }

        const replicas = {
            running: _.size(_.filter(pods,
                (pod) => pod.getEntityStatus().status === BaseSA.types.suceess.status)),
            total: _.size(pods),
        };

        this._updateMetadata({
            containers: _.flattenDeep(containers),
            replicas
        });
    }

    _onServiceAdded(svc) {
        const opt = {
            endpoint: {
                internal: svc.getClusterIP(),
                ports: svc.getPorts(),
                exposed: svc.isExposed()
            }
        };
        if (svc.isExposed() && !svc.hasEndpoints()) {
            this._updateMetadata(opt);
        }
        else if (svc.isExposed() && svc.hasEndpoints()) {
            this._updateMetadata(_.merge(opt, { endpoint: { external: svc.getEndpoints() } }));
        }
    }

    /**
     * other
     */
    _subscribeToEvents() {
        _.forOwn(eventsToExecutoresMap, (executorCandidate, eventname) => {
            if (!_.isArray(executorCandidate)) {
                this.on(eventname, this[executorCandidate]);
            } else {
                _.forEach(executorCandidate, (executor) => this.on(eventname, this[executor]));
            }
        });
    }

    _updateMetadata(data) {
        _.merge(this._metadata, data);
    }

    _updateProjectLabels(labels) {
        this._updateMetadata({ labels });
    }

}

module.exports = Project;