const _ = require('lodash');
const Request = require('kubernetes-client/backends/request');
const kubeEndpoints = require('../kube-endpoints');

const SEMVER_REGEXP = /([0-9]+)\.([0-9]+)\.([0-9]+)(?:-([0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*))?(?:\+[0-9A-Za-z-]+)?/; // valid examples: 1.22.3, 2.1.0
const SEMVER_REGEXP_MINOR_GROUP = 2;


class KubeWrapper {
    constructor(config) {
        this._request = new Request(config);

        this.init = this.init.bind(this);

        this._getVersion = this._getVersion.bind(this);
    }

    async init() {
        const version = await this._getVersion();
        this._endpoints = this._resolveEndpointsVersion(version);
    }


    getEndpoints() {
        return this._endpoints;
    }

    request(path) {
        return this._getRequestMethod(path);
    }

    _getRequestMethod(path) {
        const result = (name) => {
            return this._getRootMethods(`${path}/${name}`);
        };
        const rootMethods = this._getRootMethods(path);
        _.forEach(rootMethods, (method, name) => {
            result[name] = method;
        });

        return result;
    }

    _getRootMethods(path) {
        return [
            'GET',
            'HEAD',
            'POST',
            'PUT',
            'DELETE',
            'PATCH',
            'OPTIONS'
            ].reduce((accum, method) => {
            const name = method.toLowerCase();
            return {
                ...accum,
                [name]: (data) => {
                    return this._request.http({
                        method,
                        pathname: path,
                        ...data,
                    })
                }
            }
        }, {});

    }

    _resolveEndpointsVersion(versionData) {
        const matches = versionData.gitVersion.match(SEMVER_REGEXP);

        if (!matches) {
            return kubeEndpoints.v15;
        }
        const minor = Number(matches[SEMVER_REGEXP_MINOR_GROUP]);
        if (minor > 15) {
            return kubeEndpoints.v16;
        }
        return kubeEndpoints.v15;
    }

    async _getVersion() {
        const { body } = await this._request.http({
            method: 'GET',
            pathname: '/version'
        });
        return body;
    }
}

module.exports = KubeWrapper;
