const nock = require('nock');
const chai = require('chai');

const KubeWrapper = require('../KubeWrapper');


const expect = chai.expect;

const BASE_CLUSTER_URL = 'https://test-cluster:12345';

const TEST_KUBE_CONFIG = {
    'url': BASE_CLUSTER_URL,
    'auth': {
        'bearer': Buffer.from('test-auth')
    },
    'type': 'sat',
    'name': 'test-cluster',
    'request': {},

};

async function getMockedWrapper() {
    nock(BASE_CLUSTER_URL)
        .get('/version')
        .reply(200, {
            gitVersion: '1.16.2'
        });

    const kubeWrapper = new KubeWrapper(TEST_KUBE_CONFIG);
    await kubeWrapper.init();

    return kubeWrapper;
}

describe('KubeWrapper', () => {
    after(() => {
        nock.cleanAll();
    });

    it('should send correct requests for get', async () => {
        const kubeWrapper = await getMockedWrapper();

        const deployEndpoint = kubeWrapper.getEndpoints().secret;

        const result = [
            {
                demo: 'test-name'
            }
        ];

        nock(BASE_CLUSTER_URL)
            .get(`/${deployEndpoint.namespacePath}/demo-ns/${deployEndpoint.path}`)
            .reply(200, result);

        const response = await kubeWrapper.request(`${deployEndpoint.namespacePath}/demo-ns/${deployEndpoint.path}`).get();
        expect(response.body).to.deep.eq(result);
    });

    it('should send correct requests for post', async () => {
        const kubeWrapper = await getMockedWrapper();

        const deployEndpoint = kubeWrapper.getEndpoints().secret;

        const result = {
            demo: 'test-name'
        };
        const demoBody = { body: { test: true } };

        nock(BASE_CLUSTER_URL)
            .post(`/${deployEndpoint.namespacePath}/demo-ns/${deployEndpoint.path}`, demoBody.body)
            .reply(200, result);

        const response = await kubeWrapper
            .request(`${deployEndpoint.namespacePath}/demo-ns/${deployEndpoint.path}`)
            .post(demoBody);
        expect(response.body).to.deep.eq(result);
    });

    it('should send correct requests for delete by name', async () => {
        const kubeWrapper = await getMockedWrapper();

        const deployEndpoint = kubeWrapper.getEndpoints().secret;

        const result = {
            demo: 'test-name'
        };

        nock(BASE_CLUSTER_URL)
            .delete(`/${deployEndpoint.namespacePath}/demo-ns/${deployEndpoint.path}/test-name`)
            .reply(200, result);

        const response = await kubeWrapper.request(`${deployEndpoint.namespacePath}/demo-ns/${deployEndpoint.path}`)('test-name').delete();
        expect(response.body).to.deep.eq(result);
    });
});
