'use strict';

const _           = require('lodash');
const uuidv4      = require('uuid/v4');
const BaseBuilder = require('./../../kube-native/builder.base');
const CFError = require('cf-errors');

class DeployBuilder extends BaseBuilder {

    constructor(descriptor, opt) {
        const template = {
            kind: 'Deployment',
            metadata: {
                name: descriptor.metadata.name
            },
            spec: {
                selector: {
                    matchLabels: {
                        app: descriptor.metadata.name
                    }
                },
                strategy: {
                    type: 'RollingUpdate',
                    rollingUpdate: {
                        maxUnavailable: '50%',
                        maxSurge: '50%'
                    }
                },
                template: {
                    metadata: {
                        labels: {
                            app: descriptor.metadata.name
                        }
                    },
                    spec: {
                        terminationGracePeriodSeconds: 40,
                        restartPolicy: 'Always',
                    }
                }
            }
        };
        super(descriptor, template, opt);
    }

    _startBuild() {
        this._buildContainers() // must have
            ._buildReplicas() // must be at the end
            ._buildLabels()
            ._buildAnnotations()
            ._buildDockerImageSecrets();
    }

    _buildAnnotations() {
        this._shouldMarkAnnotations() && this.updateTemplate('spec.template.metadata.annotations', {
            forceRedeployUniqId: uuidv4(),
        });
        return this;
    }

    _getReplicas() {
        const deploymentDescriptor = this.getDescriptor();
        const replicas             = _.get(deploymentDescriptor, 'replicas', '1');
        return parseInt(replicas);
    }

    _getDockerPullSecret() {
        const deploymentDescriptor = this.getDescriptor();
        return _.get(deploymentDescriptor, 'spec.template.spec.imagePullSecrets', []);
    }

    _getResources() {
        const deploymentDescriptor = this.getDescriptor();
        return deploymentDescriptor.resources;
    }

    _getPorts() {
        const deploymentDescriptor = this.getDescriptor();
        return _.get(deploymentDescriptor, 'ports', []).map((port) => {
            return {
                containerPort: port.containerPort,
                protocol: port.protocol || 'TCP'
            }
        });
    }

    _getEnvs() {
        const deploymentDescriptor = this.getDescriptor();
        return _.get(deploymentDescriptor, 'env', []);
    }

    _getTemplateLabels() {
        return _.get(this._template, 'spec.template.metadata.labels');
    }

    _getContainers() {
        const deploymentDescriptor = this.getDescriptor();
        const containers           = _.get(deploymentDescriptor,
            'spec.template.spec.containers',
            []);
        return containers.map((container, index) => {
            if (!container.image) {
                throw new CFError({
                    message: `Deployment descriptor missing image`,
                    recognized: true,
                });
            }
            const containerTemplate = {
                image: container.image,
                imagePullPolicy: 'Always',
                name: `${_.get(this, '_descriptor.name') || `container-${index}`}`, //todo work on it
            };
            if (container.ports) {
                containerTemplate.ports = container.ports;
            }

            if (container.env) {
                containerTemplate.env = container.env;
            }

            if (container.resources) {
                containerTemplate.resources = container.resources;
            }
            return containerTemplate;
        });
    }

    _shouldBuildDockerImageSecrets() {
        return !!_.get(this.getDescriptor(), 'spec.template.spec.imagePullSecrets[0]', false);
    }

    _shouldBuildRecources() {
        return !!this.getDescriptor().resources || false;
    }

    _shouldBuildPorts() {
        return !!this.getDescriptor().ports || false;
    }

    _shouldBuildEnvs() {
        return !!this.getDescriptor().env || false;
    }

    _buildReplicas() {
        this.updateTemplate('spec.replicas', this._getReplicas());
        return this;
    }

    _buildDockerImageSecrets() {
        this._shouldBuildDockerImageSecrets() &&
        this.updateTemplate('spec.template.spec.imagePullSecrets', this._getDockerPullSecret());
        return this;
    }

    _buildResources() {
        this._shouldBuildRecources() &&
        this.updateTemplate('spec.template.spec.containers[0].resources', this._getResources());
        return this;
    }

    _buildPorts() {
        this._shouldBuildPorts() &&
        this.updateTemplate('spec.template.spec.containers[0].ports', this._getPorts());
        return this;
    }

    _buildEnvs() {
        this._shouldBuildEnvs() &&
        this.updateTemplate('spec.template.spec.containers[0].env', this._getEnvs());
        return this;
    }

    _buildContainers() {
        this.updateTemplate('spec.template.spec.containers', this._getContainers());
        return this;
    }

    _buildLabels() {
        this.shouldMarkWithLabels() && this.updateTemplate('spec.template.metadata.labels', {
            ['io.codefresh.auto-generated']: uuidv4(),
            ['io.codefresh.generated-at']: Date.now().toString()
        });
        this.updateTemplate('metadata.labels', this._getTemplateLabels());
        return this;
    }

}

module.exports = DeployBuilder;
