'use strict';

const _ = require('lodash');

const Base = require('./../../status.analyzer.base');

const supportedConditions = {
    PodScheduled: {
        value: 'PodScheduled',
        exec: '_actOnPodScheduledType'
    },
    Ready: {
        value: 'Ready',
        exec: '_actOnReadyType'
    },
    Initialized: {
        value: 'Initialized',
        exec: '_actOnInitializedType'
    },
    Unschedulable: {
        value: 'Unschedulable',
        exec: '_actOnUnschedulableType'
    }
};

const supportedStatuses = {
    True: {
        value: 'True'
    },
    False: {
        value: 'False'
    },
    Unknown: {
        value: 'Unknown'
    }
};

class PodStatusAnalyzer extends Base {

    getStatus() {
        return _.merge({}, this._givenResource.status, {
            deletionTimestampScheduled: !!_.get(this,
                '_givenResource.metadata.deletionTimestamp',
                false)
        });
    }

    _getGivenStatus() {
        return _.get(this, '_givenResource.status', {});
    }

    analyze() {
        this._finalStauts = Base.types.suceess;
        const status = this.getStatus();
        const statusConditions = status.conditions || [];
        if (status.deletionTimestampScheduled) {
            this._finalStauts  = Base.types.inProgress;
            this._extraMessage = `Pod scheduled termination at: ${_.get(this,
                '_givenResource.metadata.deletionTimestamp')}`
        }

        _.forEach(statusConditions, (condition) => {
            if(supportedConditions[condition.type]){
                this[supportedConditions[condition.type].exec](condition, status.containerStatuses || []);
            }
        });

        const res = _.merge({}, this._finalStauts, {
            givenStatus: this._getGivenStatus(),
            resouceType: this._resource
        });

        if (this._extraMessage) {
            res.message = this._extraMessage;
        }

        return res;
    }

    /**
     * When pod schedule type is "False" meaning that for some reason the scheduling wasn't successful, CPU etc
     * @param condition
     * @private
     */
    _actOnPodScheduledType(condition){
        if(condition.type === supportedConditions.PodScheduled.value && condition.status === supportedStatuses.False.value){
            this._finalStauts = Base.types.warning;
            this._extraMessage = condition.message;
        }
    }

    /**
     * Pod can be on not ready status when the image cannot be pulled for any reason
     * @param condition
     * @private
     */
    _actOnReadyType(condition, containerStatuses){
        if(condition.status === supportedStatuses.False.value) {
            const notReadyContainer = _.find(containerStatuses, { ready: false });
            const reason = _.get(notReadyContainer, `state.waiting.reason`);
            if(reason === 'ContainerCreating'){
                this._finalStauts = Base.types.inProgress;
                this._extraMessage =  reason;
            } else {
                this._finalStauts = Base.types.warning;
                this._extraMessage = _.get(notReadyContainer, `state.waiting.message`) || _.get(notReadyContainer, `state.terminated.message`);
            }

        }
    }

    _actOnInitializedType(condition){
        if(condition.status === supportedStatuses.False.value){
            this._finalStauts = Base.types.failed;
        }
    }


    _actOnUnschedulableType(){
        this._finalStauts = Base.types.failed;
    }


}


module.exports = PodStatusAnalyzer;