'use strict';

const _            = require('lodash');
const Promise      = require('bluebird');
const ResourсeController = require('../resource.base.controller');
const StatusAnalyzer     = require('../../status.analyzer.base');

class ServiceController extends ResourсeController {
    /**
     * Analyze the total status of the svc,deploy,rs,pod set
     * @param svc
     * @param deployment
     * @param rsArray
     * @param podArray
     * @return {{type: string, resources: {svc: *, deployment: *, replicaSets, pods}}
     * @private
     */
    _analyzeStatus({ svc, deployment, rsArray, podArray }) {
        const status = {
            type: StatusAnalyzer.types.unknown.status,
            resources: {
                svc: svc.getEntityStatus(),
                deployment: deployment && deployment.getEntityStatus(),
                replicaSets: rsArray && rsArray.map(rs => rs.getEntityStatus()),
                pods: podArray && podArray.map(pod => pod.getEntityStatus())
            }
        };

        if (svc.getEntityStatus().status !== StatusAnalyzer.types.suceess.status) {
            status.type    = StatusAnalyzer.types.failed;
            status.message = 'Kubernetes service not available';
            return status;
        }

        if (_.size(rsArray) > 1) {
            let rs = _.find(rsArray, (obj) => {
                return obj.getEntityStatus().status !== StatusAnalyzer.types.suceess.status;
            });
            if (!rs) {
                rs = _.head(rsArray);
            }
            const key          = rs.getLabelByKey('pod-template-hash');
            const pods         = _.filter(podArray, (obj) => {
                return obj.getLabelByKey('pod-template-hash') === key;
            });
            const pod          = _.head(pods);
            const podStatusObj = pod.getEntityStatus();
            status.type        = podStatusObj.status;
            status.message     = podStatusObj.message;

        } else {
            const pod = _.find(podArray,
                (po) => po.getEntityStatus().status !== StatusAnalyzer.types.suceess.status);
            if (pod) {
                const podStatusObj = pod.getEntityStatus();
                status.type        = podStatusObj.status;
                status.message     = podStatusObj.message;
            } else {
                status.type = StatusAnalyzer.types.suceess.status;
            }
        }
        return status;
    }

    _getRelatedResources({ selector, namespace }) {
        const kubeManager          = this._kubeManager;
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const replicasetController = kubeManager.getReplicasetController.bind(kubeManager);
        const podController        = kubeManager.getPodController.bind(kubeManager);
        return Promise.all([
            deploymentController(namespace)
                .get({ labelSelector: selector })
                .then(_.head),
            replicasetController(namespace)
                .get({ labelSelector: selector })
                .map(rs => rs.getReplicas() !== 0 ? rs : undefined)
                .then(_.compact),
            podController(namespace)
                .get({ labelSelector: selector }),
        ]);
    }

    async describeFull(name, namespace) {
        const kubeManager = this._kubeManager;
        const serviceController = kubeManager.getServiceController.bind(kubeManager);
        const namespaces = namespace
            ? [namespace]
            : await kubeManager.getNamespaceController().get().map(ns => ns.getName());

        return Promise.map((namespaces), (ns) => {

            return serviceController(ns)
                .describe(name)
                .then((svc) => {
                    const serviceSelector = svc.getServiceSelector();
                    if (!_.isEmpty(serviceSelector)) {
                        return Promise.map(_.toPairs(serviceSelector), ([key, value]) => `${key}=${value}`)
                            .then(_.head)
                            .then(selector => this._getRelatedResources({ selector, namespace: ns }))
                            .then(([deployment, rsArray, podArray]) => {
                                const analyzedStatus = this._analyzeStatus({ svc, deployment, rsArray, podArray });
                                return _.merge(svc.getBasicData(),
                                    {
                                        globalStatus: analyzedStatus,
                                        deployment: deployment && deployment.getBasicData()
                                    })
                            })
                            .catch((err) => {
                                // todo: report to new relic
                                const analyzedStatus = this._analyzeStatus({ svc });
                                return _.merge(svc.getBasicData(), { globalStatus: analyzedStatus });
                            });
                    }

                    const analyzedStatus = this._analyzeStatus({ svc });
                    return _.merge(svc.getBasicData(), { globalStatus: analyzedStatus });
                });
        })
            .then(_.flattenDeep)
            .then(_.head);
    }
}

module.exports = ServiceController;
