'use strict';

const chai       = require('chai');
const sinonChai  = require('sinon-chai');
const _          = require('lodash');
const SvcBuilder = require('./../svc.builder');

const expect = chai.expect;
chai.use(sinonChai);


describe('Svc builder unit test', () => {

    describe('construct', () => {
        const tests = [
            {
                title: 'simple',
                descriptor: {
                    metadata: {
                        name: 'fake'
                    }
                },
                expectedObject: {
                    "apiVersion": "v1",
                    "kind": "Service",
                    "metadata": {
                        "name": "fake"
                    },
                    "spec": {
                        "selector": {
                            "app": "fake"
                        },
                        "type": "ClusterIP"
                    }
                },
            },
            {
                title: 'with LoadBalancer',
                descriptor: {
                    metadata: {
                        name: 'fake'
                    },
                    spec: {
                        type: 'LoadBalancer'
                    }
                },
                expectedObject: {
                    "apiVersion": "v1",
                    "kind": "Service",
                    "metadata": {
                        "name": "fake"
                    },
                    "spec": {
                        "selector": {
                            "app": "fake"
                        },
                        "type": "LoadBalancer"
                    }
                }
            },
            {
                title: 'with ports',
                descriptor: {
                    metadata: {
                        name: 'fake'
                    },
                    spec: {
                        ports: [{
                            name: 'port-1',
                            port: 80,
                            targetPort: 9000,
                            protocol: 'TCP'
                        }
                        ]
                    }
                },
                expectedObject: {
                    "apiVersion": "v1",
                    "kind": "Service",
                    "metadata": {
                        "name": "fake"
                    },
                    "spec": {
                        "ports": [
                            {
                                "name": "port-1",
                                "port": 80,
                                "protocol": "TCP",
                                "targetPort": 9000
                            }
                        ],
                        "selector": {
                            "app": "fake"
                        },
                        "type": "ClusterIP"
                    }
                }
            },
            {
                title: 'with labels',
                descriptor: {
                    metadata: {
                        name: 'fake'
                    },
                },
                markWithLabels: true,
                keys: [{
                    path: 'metadata.labels',
                    include: ['io.codefresh.auto-generated', 'io.codefresh.generated-at']
                }
                ]
            }
        ];
        tests.map((test) => {
            it(`with ${test.title}`, () => {
                const finalObject = new SvcBuilder(test.descriptor,
                    { markWithLabels: test.markWithLabels || false }).build();
                if (!test.markWithLabels) {
                    expect(finalObject).to.be.deep.equal(test.expectedObject);
                } else {
                    _.forEach(test.keys, (key) => {
                        expect(_.get(finalObject, key.path)).to.have.keys(key.include);
                    });
                }
            });
        });
    });

});